###GSTAT###
#load applicable libraries
library(gstat)
library(sp)
library(tcltk)
library(plyr)
library(dostats)
library(SiZer)


#JUNE
#set working directory to where all text files are listed
setwd("C:/...")

#Make a list of the text file names
files = list.files("C:/...", pattern="*.txt", full.names=FALSE)

#Write the order of the list to a csv in case
write.csv(files, file="Junefiles.csv")

#set up an empty dataframe with appropriate spots for data
name.range.nugget.sill=data.frame(depth=rep(0,5),platename=rep(0,5),range=rep(0,5), nugget=rep(0,5),sill=rep(0,5),slopepv=rep(0,5),curvepv=rep(0,5),sillaccept=rep(0,5))

#set up an empty dataframe
allplates=NULL

#for each file in the list of files in that folder,
for (i in 1:length(files)) {
    
#read the file in, one file at a time
  hetero <- read.table(files[i], header=FALSE, stringsAsFactors=FALSE)
#only use the top 30% of the plate 
  hetero03=hetero[0:(length(hetero$V1)*0.3),]
  
#for each one file you have read in, repeat the following five times:
for (j in 1:5) {
#subset each file for 5000 random pixels and do not replace the sampled row back into the dataframe
   hetero.subset=(hetero03[sample(1:nrow(hetero03), 5000, replace=FALSE),])


#Get a map of your coordinates, and find the range: x goes from pixel 1 through pixel n. Turn them into a grid.
coordinates(hetero.subset) = ~ V1 + V2
x.range <- as.integer(range(hetero.subset@coords[,1]))
y.range <- as.integer(range(hetero.subset@coords[,2]))
gridded(hetero.subset)=TRUE

#Create a gstat object out of the data points and plot the variogram of your data if you want
g=gstat(formula=V3~V1+V2, data=hetero.subset)
#plot(variogram(g), main=files[i], pch=19)

#assign the variogram to an object
v=variogram(g, width=50)
##variogram.fit <- fit.variogram(v, model=vgm(10,"Lin",50,100),fit.sills=TRUE, fit.ranges=TRUE, fit.method=TRUE)
##plot(v, model=variogram.fit, pch=19)

#get the p value of the slope fo the ine and the "curve" part of the line to see if there is a significant difference between the two parts of the line
slopepv <- summary(lm(v$gamma~v$dist+I(v$dist^2)))$coefficients[2,4] 
curvepv <- summary(lm(v$gamma~v$dist+I(v$dist^2)))$coefficients[3,4] 

#create a dataframe of exracted parameters from the variogram
xy=data.frame(distance=v$dist,gamma=v$gamma)
#plot a linear version of the variogram with a distinct changepoint
plot(piecewise.linear(xy$distance,xy$gamma), direction=pi/4)


#assign the line/model to an object
piecewise=piecewise.linear(xy$distance,xy$gamma)
#obtain the location of the changepoint
sill=as.numeric(predict(piecewise,piecewise$change.point))
#gather the name of the plate, the nugget parameter, the range and the slope
platename=files[i]
nugget=xy[1,2]
range=piecewise.linear$change.point
slope <- summary(lm(v$gamma~v$dist))$coefficients[1,1] 

#put each parameter into the dataframe we created earlier
#If the curve p value is <0.05, use the sill, otherwise use the nugget 
name.range.nugget.sill[j,]=data.frame(depth=03,platename=platename,range=range,nugget=nugget,sill=sill,slopepv=slopepv,curvepv=curvepv,slope=slope, stringsAsFactors = F)
name.range.nugget.sill$sillaccept <- ifelse(name.range.nugget.sill$curvepv<0.05,name.range.nugget.sill$sill,name.range.nugget.sill$nugget)
}
#gather all data from all plates and reps into one dataframe
allplates=rbind(allplates,name.range.nugget.sill)
}


#write into a csv for safekeeping
write.csv(allplates, file="Juneallplates_trial.csv")   




#If you want to use a different depth range of the plate, change the fraction value in line 33